@@ -145,7 +145,7 @@ go back to ((<Index>))
     NArrayMiss.new(OBJECT,*arg)
   end
   def self.[](*arg)
-    NArrayMiss.to_nam(*arg)
+    NArrayMiss.to_nam(NArray[*arg])
   end
   def self.to_nam_no_dup(*arg)
     if arg.length > 2 || arg.length==0 then
@@ -153,7 +153,7 @@ go back to ((<Index>))
     end
 
     array = arg[0]
-    if array.class < Numeric then array = NArray[array] end
+    if Numeric===array then array = NArray[array] end
     if array.class == Array then array = NArray.to_na(array) end
     if array.class != NArray then
       raise("argument must be Numeric, NArray or Array")
@@ -161,12 +161,9 @@ go back to ((<Index>))
 
     if arg.length==2 then
       mask = arg[1]
-      if mask.class < Numeric then mask = [mask] end
-      if mask.class == TrueClass then mask = [1] end
+      if Numeric===mask then mask = [mask] end
       if mask.class == Array then
-	tmp = NArray.byte(mask.length)
-	tmp[]=mask
-	mask = tmp
+	mask = NArray.to_na(mask).ne(0)
       end
       if mask.class == FalseClass then
 	mask = NArray.byte(*array.shape)
@@ -177,16 +174,20 @@ go back to ((<Index>))
       if !(mask.class==NArray && mask.typecode==BYTE) then
 	  raise("mask must be Numeric, Array, true, false or NArray(byte)")
       end
+      if mask.length!=array.length
+        raise "mask.length must be same as array.length"
+      end
     else
       mask = NArray.byte(*array.shape).fill(1)
     end
     __new__(array,mask)
   end
   def self.to_nam(*arg)
-    if arg[0].class!=TrueClass && arg[0].class!=FalseClass then
-      arg[0] = arg[0].dup
+    if !Numeric===arg[0] && arg[0].class!=Array && arg[0].class!=NArray
+      raise "first argument must be Numeric, NArray or Array"
     end
-    if arg.length==2 && arg[1].class!=TrueClass && arg[1].class!=FalseClass then
+    arg[0] = arg[0].dup if !Numeric===arg[0]
+    if arg.length==2 && !Numeric===arg[1] && arg[1].class!=TrueClass && arg[1].class!=FalseClass then
       arg[1] = arg[1].dup
     end
     NArrayMiss.to_nam_no_dup(*arg)
@@ -259,10 +260,11 @@ go back to ((<Index>))
 =end
 
   def [](*arg)
-    if arg.length==1 then
-      return @array[arg[0]]
+    obj = @array[*arg]
+    if Numeric===obj
+      return obj
     else
-      return NArrayMiss.to_nam_no_dup(@array[*arg],@mask[*arg])
+      return NArrayMiss.to_nam_no_dup(obj,@mask[*arg])
     end
   end
   def slice(*arg)
@@ -562,15 +564,27 @@ go back to ((<Index>))
     eval(routine,nil,__FILE__,__LINE__+1)
   end
   def mean(*arg)
-    if arg.length!=0 && arg[-1].class==Hash then arg2=arg[0..-2]
-    else arg2=arg end
-    mask = @mask.to_type(NArray::INT).sum(*arg2)
-    if mask.class == NArray then
-      mask = NArrayMiss.to_nam(mask,mask.ne(0))
-      return self.sum(*arg)/mask
+    min_count = 1
+    options=["min_count"]
+    if arg.length!=0 && arg[-1].class==Hash then
+      option = arg[-1]
+      arg2=arg[0..-2]
+      option.each_key{|key|
+	if !options.index(key) then
+	  raise(ArgumentError,key+" option is not exist")
+	end
+      }
+      min_count = option["min_count"]
     else
-      if mask!=0 then
-	return self.sum(*arg)/mask
+      arg2=arg
+    end
+    count = @mask.to_type(NArray::INT).sum(*arg2)
+    if count.class == NArray then
+      count = NArrayMiss.to_nam(count,count.ge(min_count))
+      return self.sum(*arg)/count
+    else
+      if count >= min_count then
+	return self.sum(*arg)/count
       else
 	return NArrayMiss.to_nam([self.sum(*arg)],NArray.byte(1))
       end
@@ -1089,7 +1103,7 @@ go back to ((<Index>))
   end
 
   def coerce(x)
-    if x.class<Numeric then
+    if Numeric===x then
       return [NArrayMiss.new(NArray[x].typecode,*self.shape).fill(x),self]
     elsif x.class==Array || x.class==NArray then
       return [NArrayMiss.to_nam(x), self]
@@ -1103,55 +1117,63 @@ go back to ((<Index>))
 #    "array -> " + @array.inspect + "\nmask  -> " + @mask.inspect
     count_line = 0
     max_line = 10
-    max_col = 77
+    max_col = 80
     sep = ", "
     const = Hash.new
     NArray.constants.each{|c| const[NArray.const_get(c)] = c}
-    print("NArrayMiss."+const[typecode].downcase+"("+shape.join(",")+"):")
+    str_ret = "NArrayMiss."+const[typecode].downcase+"("+shape.join(",")+"):"
     if rank == 0 then
-      print(" []")
-      return
+      str_ret += " []"
+      return str_ret
     else
-      print("\n")
+       str_ret += "\n"
     end
     str = ""
     index = Array.new(rank,0)
     index[0] = true
     i = 1
-    (rank-1).times{ print("[ ") }
+    (rank-1).times{ str_ret += "[ " }
     while(true)
-      i.times{ print("[ ") }
+      i.times{ str_ret += "[ " }
 
       str = @array[*index].inspect
       ary = str[str.index("[")+1..str.index("]")-1].strip.split(/\s*,\s*/)
-      miss = @mask[*index].where2[1].to_a
+      miss = @mask[*index].where2[1]
+      miss = miss[miss<ary.length].to_a
+      if ary[-1]=="..." && miss[-1]==ary.length-1 then miss.pop end
       for j in miss
 	ary[j] = "-"
       end
-      print( ary.join(", ") )
+      while ( rank*4+ary.join(", ").length > max_col )
+        ary.pop
+        ary[-1] = "..."
+      end
+      str_ret += ary.join(", ")
       i = 1
       while (i<rank)
 	if index[i]<shape[i]-1 then
-	  print(" ]"+sep+"\n")
+	  str_ret += " ]"+sep+"\n"
 	  count_line += 1
 	  index[i] += 1
 	  break
 	else
-	  print(" ]")
+	  str_ret += " ]"
 	  index[i] = 0
 	  i += 1
 	end
       end
 
       if i>=rank then
-	return
+	str_ret += " ]"
+	return str_ret
       elsif count_line>=max_line then
-	print(" ...")
-	return
+	str_ret += " ..."
+	return str_ret
       end
 
       (rank-i).times{ print("  ") }
     end
+    return str_ret
   end
 
 
@@ -1171,7 +1193,7 @@ go back to ((<Index>))
   end
   def routine1(arg,dummy)
     flag=true
-    if arg.class < Numeric then
+    if Numeric===arg then
       term1 = @array
       term2 = arg
       mask = true
@@ -1207,7 +1229,7 @@ go back to ((<Index>))
 	obj.set_mask(mask.eq(2))
       end
       obj.set_without_validation(@mask.where2[1],@array[@mask.where2[1]])
-      if obj.length==1 && obj.get_mask![0]==1 then
+      if Numeric===obj && obj.get_mask![0]==1 then
 	return obj.get_array[0]
       else
 	return obj
@@ -1232,7 +1254,7 @@ module NMMath
   for operator in func1
     eval <<-EOL,nil,__FILE__,__LINE__+1
     def #{operator}(x)
-      if x.class < Numeric || x.class==Array || x.class==NArray then
+      if Numeric===x || x.class==Array || x.class==NArray then
 	NMath::#{operator}(x)
       elsif x.class == NArrayMiss then
 	obj = NArrayMiss.new(x.typecode,*x.shape)
@@ -1250,13 +1272,13 @@ module NMMath
   for operator in func2
     eval <<-EOL,nil,__FILE__,__LINE__+1
     def #{operator}(x,y)
-      if x.class < Numeric || x.class==Array || x.class==NArray then
+      if Numeric===x || x.class==Array || x.class==NArray then
 	mask1 = nil
       elsif x.class == NArrayMiss then
 	obj = NArrayMiss.new(x.typecode,*x.shape)
 	mask1 = x.get_mask!
       end
-      if y.class < Numeric || y.class==Array || y.class==NArray then
+      if Numeric===y || y.class==Array || y.class==NArray then
 	mask2 = nil
       elsif y.class == NArrayMiss then
 	obj = NArrayMiss.new(y.typecode,*y.shape)
